



StructuredBuffer<uint> Input ;

StructuredBuffer<uint> tempB;

RWStructuredBuffer<uint> Result : BACKBUFFER;




bool getBit(uint i, uint n) {
	return ((n >> i) & 1) == 1;
}

#define GROUP_THREADS 16
groupshared uint o[GROUP_THREADS];
groupshared uint e[GROUP_THREADS];
groupshared uint f[GROUP_THREADS];
//groupshared uint t[GROUP_THREADS];
groupshared uint d[GROUP_THREADS];
groupshared uint totalFalses;

// SV_DispatchThreadID - index of the thread within the entire dispatch in each dimension: x - 0..x - 1; y - 0..y - 1; z - 0..z - 1
// SV_GroupID - index of a thread group in the dispatch ? for example, calling Dispatch(2,1,1) results in possible values of 0,0,0 and 1,0,0, varying from 0 to (numthreadsX * numthreadsY * numThreadsZ) ? 1
// SV_GroupThreadID - 3D version of SV_GroupIndex - if you specified numthreads(3,2,1), possible values for the SV_GroupThreadID input value have the range of values (0?2,0?1,0)
// SV_GroupIndex - index of a thread within a thread group

[numthreads(16, 1, 1)]
void CS_Radix(
	uint3 Gid  : SV_GroupID,
	uint3 DTid : SV_DispatchThreadID,
	uint3 GTid : SV_GroupThreadID,
	uint  GI : SV_GroupIndex)
{


	// index out of structured buffer input
	uint idx = DTid.x;

	// populate the o array with the uint versions of this block
	o[GI] = Input[idx];


	// loop through each bit
	[unroll(32)]
	for (int n = 0; n < 32; n++) {

		// e is 1 where the nth bit is 0.
		e[GI] = getBit(n, o[GI]) == 0;

		GroupMemoryBarrierWithGroupSync(); // wait for e to be populated so we can random access it

		if (GI != 0) {
			f[GI] = e[GI - 1];
		}
		else {
			f[GI] = 0;
		}

		GroupMemoryBarrierWithGroupSync(); // wait for f to be populated before we loop on it

		// Scan Operation (AKA Prefix Sum)
		[unroll(int(log2(GROUP_THREADS)))]
		for (uint i = 1; i < GROUP_THREADS; i <<= 1) { //for n = 0 .. log2(N), i =  2^n
			uint temp;
			if (GI > i) {
				temp = f[GI] + f[GI-i];
			}
			else {
				temp = f[GI];
			}
			GroupMemoryBarrierWithGroupSync();
			f[GI] = temp;
			GroupMemoryBarrierWithGroupSync();

		}

		// Sum up the falses
		if (GI == 0) {
			totalFalses = e[GROUP_THREADS - 1] + f[GROUP_THREADS - 1];
		}

		GroupMemoryBarrierWithGroupSync(); // wait for thread 0 to finish

		// we now construct t on the fly

		// d contains the destination indexes for all the bits
		d[GI] = e[GI] ? f[GI] : GI - f[GI] + totalFalses;

		// get the variable
		uint temp = o[GI];

		GroupMemoryBarrierWithGroupSync(); // read-before-write

		// rewrite o
	    o[d[GI]] = temp; // This line gives the error compiling.

		GroupMemoryBarrierWithGroupSync(); // wait for o to be fully populated

	}


	
	Result[DTid.x] = (o[GI]) ;


}


technique11 CS_RadixT
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CS_Radix() ) );
	}
}